$supportUpdaterMigration = true;

// Get custom .BLS callbacks ready
// NOTE: Brick QuotaObjects are handled internally when the following callbacks are executed.


//=======

function BLS_checkBrickAudioInstalled (%isServerObject, %brick, %labelName)
{
	return isObject (%brick.AudioEmitter) && %brick.AudioEmitter.getProfileId ().uiName !$= "";
}

function BLS_onBrickAudioInstalled (%isServerObject, %brick, %labelName, %uiName)
{
	if (!%isServerObject)
		return;
	
	%musicDB = $uiNameTable_Music[%uiName];
	%brick.setSound (%musicDB);
}

//=======

function BLS_checkBrickVehicleSpawnInstalled (%isServerObject, %brick, %labelName)
{
	return isObject (%brick.vehicleSpawnMarker) && %brick.vehicleSpawnMarker.getUIName () !$= "";
}

function BLS_onVehicleSpawnInstalled (%isServerObject, %brick, %labelName, %uiName, %reColorVehicle)
{
	if (!%isServerObject)
		return;
	
	%vehDB = $uiNameTable_Vehicle[%uiName];
	
	// Ensure the datablock exists
	%brick.setVehicle (%vehDB);
	%brick.setReColorVehicle (%reColorVehicle);
}

//=======

function BLS_BotHole_checkBotInstalled (%isServerObject, %brick, %labelName)
{
	return isObject (%brick.hBot) && %brick.hBot.getDataBlock ().uiName !$= "";
}

function BLS_BotHole_onBotInstalled (%isServerObject, %brick, %labelName, %uiName)
{
	if (!%isServerObject)
		return;
	
	%vehDB = $uiNameTable_Player[%brick.hBot.getDataBlock ().uiName];
	
	// Ensure the datablock exists
	%brick.setBot (%vehDB);
}

//=======

// Register common spawns

// BOT HOLES
%LastBType = BLS_RegisterBrickType("isBotHole", "1");
%LastLabel = BLS_AddSaveLabel(%LastBType, "BOT", "BLS_BotHole_checkBotInstalled", "BLS_BotHole_onBotInstalled");
	     BLS_AddSaveField(%LastBType, %LastLabel, "uiname", "hBot->getDataBlock()->uiName");

// VEHICLE SPAWNS
%LastBType = BLS_RegisterBrickType("specialBrickType", "VehicleSpawn");
%LastLabel = BLS_AddSaveLabel(%LastBType, "VEHICLE", "BLS_checkBrickVehicleSpawnInstalled", "BLS_onVehicleSpawnInstalled");
	     BLS_AddSaveField(%LastBType, %LastLabel, "uiname", "VehicleSpawnMarker->getUiName()");
	     BLS_AddSaveField(%LastBType, %LastLabel, "bool",   "VehicleSpawnMarker->getReColorVehicle()");

// MUSIC BRICKS
%LastBType = BLS_RegisterBrickType("specialBrickType", "Sound");
%LastLabel = BLS_AddSaveLabel(%LastBType, "AUDIOEMITTER", "BLS_checkBrickAudioInstalled", "BLS_onBrickAudioInstalled");
	     BLS_AddSaveField(%LastBType, %LastLabel, "uiname", "AudioEmitter->getProfileId()->uiName");
	     BLS_AddSaveField(%LastBType, %LastLabel, "float", "AudioEmitter->getPitch()");

function StartSelectedDemo2()
{
   // first unit is filename
   %sel = RecordingsDlgList.getSelectedId();
   %rowText = RecordingsDlgList.getRowTextById(%sel);

   %file = "base/recordings/demo000.rec";

   new GameConnection(ServerConnection);
   RootGroup.add(ServerConnection);

   if (ServerConnection.playDemo("base/recordings/demo000.rec"))
   {
      Canvas.popDialog(RecordingsDlg);
      ServerConnection.prepDemoPlayback();
   }
   else 
   {
      MessageBoxOK("Playback Failed", "Demo playback failed for file '" @ %file @ "'.");
      if (isObject(ServerConnection)) {
         ServerConnection.delete();
      }
   }
}

function formatTimeDifference(%time, %default)
{
	if (%time == 0 && %default == 0)
		return "no time";
	
	%a[-1+%aC++] = %time % 60 @ " second" @ ((%time % 60) == 1 ? "" : "s");
	if (%default >= 1 || %time >= 60)
	{
		if (%default < 1)
			%time = mFloor(%time / 60);
		
		%a[-1+%aC++] = %time % 60 @ " minute" @ ((%time % 60) == 1 ? "" : "s");
		if (%default >= 2 || %time >= 60)
		{
			if (%default < 2)
				%time = mFloor(%time / 60);
			
			%a[-1+%aC++] = %time % 24 @ " hour" @ ((%time % 24) == 1 ? "" : "s");
			if (%default >= 3 || %time >= 24)
			{
				if (%default < 3)
					%time = mFloor(%time / 24);
				
				%a[-1+%aC++] = %time % 7 @ " day" @ ((%time % 7) == 1 ? "" : "s");
				if (%default >= 4 || %time >= 07)
				{
					if (%default < 4)
						%time = mFloor(%time / 7);
					
					%a[-1+%aC++] = %time % 05 @ " week" @ ((%time % 05) == 1 ? "" : "s");
					if (%default >= 5 || %time >= 5)
					{
						if (%default < 5)
							%time = mFloor(%time / 5);
						
						%a[-1+%aC++] = %time % 12 @ " month" @ ((%time % 12) == 1 ? "" : "s");
						if (%default == 6 || %time >= 12)
						{
							%time = mFloor(%time / 12);
							%a[-1+%aC++] = %time % 12 @ " year" @ ((%time % 12) == 1 ? "" : "s");
						}
					}
				}
			}
		}
	}
	
	while (firstWord(%a[0]) $= "0")
	{
		for(%i = 0; %i < %aC; %i++)
			%a[%i-1] = %a[%i];
		
		%aC--;
		%default--;
	}
	
	%wCount = -1;
	%last   = %aC-1;
	for (%i = 0; %i < %aC; %i++)
		if ((%aC - 1) - %i < %default)
			%last--;
	
	for (%i = 0; %i < %aC; %i++)
	{
		%str = %a[(%aC-1)-%i];
		if ((%aC - 1) - %i < %default)
			continue;
		
		if (firstWord(%str) $= "0")
		{
			%last--;
			continue;
		}
		
		%wCount++;
		%ret = (%wCount == 0 ? "" : (%wCount == %last ? %ret @ " and " : %ret @ ", ")) @ %str;
	}
	
	return %ret;
}

function formatDateTime(%dT)
{
	%dT  = strReplace(%dT, " ", "\t");
	%dT  = strReplace(%dT, "/", " ");
	%dT  = strReplace(%dT, ":", " ");
	
	// Time
	%t_h = getWord(getField(%dT, 1), 0);
	%t_m = getWord(getField(%dT, 1), 1);
	%t_s = getWord(getField(%dT, 1), 2);
	
	%d_y = strReplace(ltrim(strReplace(getWord(getField(%dT, 0), 2), "0", " ")), " ", "0");
	if (strLen (%d_y) < 2)
		%d_y = "0" @ %d_y;
	
	// Date
	%d_m = strReplace(ltrim(strReplace(getWord(getField(%dT, 0), 0), "0", " ")), " ", "0");
	%d_d = strReplace(ltrim(strReplace(getWord(getField(%dT, 0), 1), "0", " ")), " ", "0");
	%d_y = "20" @ %d_y;
	
	// Initialize tables
	%monthTable = "January February March April May June July August September October November December";
	
	// Create the return string
	%ret =        getWord(%monthTable, mClamp(%d_m-1, 0, getWordCount(%monthTable))) @ " ";
	%ret = %ret @ %d_d @ ", ";
	%ret = %ret @ %d_y @ " at ";
	%ret = %ret @ (%t_h > 12 ? %t_h-12 : %t_h) @ ":" @ %t_m @ ":" @ %t_s SPC (%t_h > 12 ? "PM" : "AM");
	
	// Done!
	return %ret;
}

function mmc_getBest (%mem, %typeList, %byteNumType, %useCommas, %floatLen)
{
	%realList    = "B KB MB GB TB PB EB ZB YB";
	%byteNumType = (%byteNumType $= "" ? "0" : mClamp (atoi (%byteNumType), 0, 1));
	%bytePerType = (%byteNumType == 0 ? 1024 : 1000);
	%useCommas   = (%useCommas $= "" ? 1 : (atoi (%useCommas) != 0));
	%floatLen    = (%floatLen $= "" ? 2 : atoi (%floatLen));
	%typeList    = (%typeList $= "" ? %realList : trim (strUpr (%typeList)));
	
	// Begin conversions
	%startingMem = %mem;
	%suf         = getWord (%typeList, 0);
	
	// Loop through all remaining conversions
	for (%i = 0; %i < getWordCount (%typeList); %i++)
	{
		%type   = getWord (%typeList, %i);
		%tmpMem = %startingMem;
		%failed = false;
		
		// Perform the conversion
		for (%j = 0; %j < getWordCount (%realList); %j++)
		{
			if (%type $= getWord (%realList, %j))
				break;
			
			if (%i != 0 && %tmpMem < %bytePerType)
			{
				%failed = true;
				break;
			}
			
			%tmpMem /= %bytePerType;
		}
		
		if (!%failed)
		{
			%suf = %type;
			%mem = %tmpMem;
		}
	}
	
	// Determine how to return the result
	if (%useCommas)
		return getCommaNumber (mFloatLength (%mem, %floatLen)) SPC %suf;
	
	return mFloatLength (%mem, %floatLen) SPC %suf;
}