function BLRS_SaveUINames ()
{
	BLRS_LoadConversionTable ();
	
	// Ready some statistics
	%count = 0;
	
	// Loop through all datablocks
	for (%i = 0; %i < getDataBlockGroupSize (); %i++)
	{
		%db = getDataBlock (%i);
		
		// Only fetch datablocks we care about.
		if (%db.getClassName () $= "fxDTSBrickData" && %db.uiName !$= "" && %db.brickFile !$= "")
		{
			// Skip ones we've already got
			if ($MMBuildShowcase::UINameTable["brick", %db.uiName] !$= "")
				continue;
			
			// Brick File Name
			%data =
					%db.brickFile			NL
					%db.brickSizeX			NL
					%db.brickSizeY			NL
					%db.brickSizeZ;
			
//			$MMBuildShowcase::UINameTable["brick", %db.uiName]                 = %data;
			$MMBuildShowcase::UINameTable["brick", getStringCRC (%db.uiName)] = %data;
			%count++;
		}
		else if (%db.getClassName () $= "ParticleEmitterData" && %db.uiName !$= "" && %db.particles !$= "" && isObject (getWord (%db.particles, 0)))
		{
			// Skip ones we've already got
			if ($MMBuildShowcase::UINameTable["particle", %db.uiName] !$= "")
				continue;
			
			%minTime = 0;
			%maxTime = 0;
			for (%j = 0; %j < 4; %j++)
			{
				if (%partDB.times[%minTime] > %partDB.times[%j])
					%minTime = %j;
				if (%partDB.times[%maxTime] < %partDB.times[%j])
					%maxTime = %j;
			}
			
			%partDB = getWord (%db.particles, 0);
			$MMBuildShowcase::UINameTable["particle", %db.uiName] =
					%partDB.textureName		NL
					%partDB.dragCoefficient		NL
					%partDB.gravityCoefficient	NL
					%partDB.lifetimeMS		NL
					%partDB.colors[%minTime]	NL
					%partDB.colors[%maxTime]	NL
					%partDB.sizes[%minTime]		NL
					%partDB.sizes[%maxTime]		NL
					%db.ejectionVelocity		NL
					%db.ejectionPeriodMS		NL
					%db.useEmitterColors		NL
					%db.velocityVariance;
			%count++;
		}
		else if (%db.getClassName () $= "ItemData" && %db.uiName !$= "" && %db.shapeFile !$= "")
		{
			// Skip ones we've already got
			if ($MMBuildShowcase::UINameTable["item", %db.uiName] !$= "")
				continue;
			
			$MMBuildShowcase::UINameTable["item", %db.uiName] =
					%db.shapeFile			NL
					%db.doColorShift		NL
					%db.colorShiftColor;
			%count++;
		}
	}
	
	echo ("Got " @ %count @ " new brick datablock" @ (%count == 1 ? "" : "s") @ " for the bls to blrs converter.");
	
	export ("$MMBuildShowcase::UINameTable*", "config/client/buildShowcase/uiNameTable.cs", false, "$MMBuildShowcase::UINameTable");
	deleteVariables ("$MMBuildShowcase::UINameTable*");
}

function BLRS_LoadConversionTable ()
{
	if (!isFile ("config/client/buildShowcase/uiNameTable.cs"))
		exec ("Add-Ons/Client_MainMenuBuildShowcase/data/defaultUINameTable.cs");
	else
		exec ("config/client/buildShowcase/uiNameTable.cs");
}

function BLRS_ConstructData (%SO, %type, %uiName, %rawLine)
{
	switch$(%type)
	{
		case "brick":
//			%data = $MMBuildShowcase::UINameTable["brick", %uiName];
			%data = $MMBuildShowcase::UINameTable["brick", getStringCRC (%uiName)];
			
			if (%data $= "")
			{
				error ("ERROR: bls2blrs() - Failed to find brick datablock with uiname \"" @ %uiName @ "\"");
				%SO.lastBrickUIName = "";
				setclipboard(%uiname);
				return;
			}
			
			%brickFile  = getRecord (%data, 0);
			%brickSizeX = getRecord (%data, 0);
			%brickSizeY = getRecord (%data, 1);
			%brickSizeZ = getRecord (%data, 2);
			
			// Get just the brick attributes from the raw line
			%rawLine = getSubStr (%rawLine, strPos (%rawLine, "\"") + 2, strLen (%rawLine));
			
			// Configure last brick properties
			%SO.lastBrickUIName = %uiName;
			%SO.lastBrickPos    = getWords (%rawLine, 0, 2);
			%SO.lastBrickColor  = %SO.colorSet[getWord (%rawLine, 5)];
			%brickBounds        = BoxCreate (%SO.lastBrickPos, (%brickSizeX / 2) SPC (%brickSizeY / 2) SPC (%brickSizeZ / 2));
			%SO.boundsCenter    = VectorAdd (%SO.boundsCenter, %SO.lastBrickPos);
			%SO.validBrickCount++;
			
			// Expand our bounding box
			if (%SO.bounds $= "0 0 0 0 0 0")
			{
				%SO.bounds = %brickBounds;
			}
			else
			{
				%SO.bounds = BoxEncapsulate (%SO.bounds, %brickBounds);
			}
			
			// Write the data
			%SO.writeLine (%brickFile @ "\" " @ %rawLine);
			
		case "emitter":
			%data = $MMBuildShowcase::UINameTable["particle", %uiName];
			
			if (%data $= "")
			{
				error ("ERROR: bls2blrs() - Failed to find particle datablock with uiname \"" @ %uiName @ "\"");
				return;
			}
			
			%brickData = $MMBuildShowcase::UINameTable["brick", %SO.lastBrickUIName];
			if (%brickData $= "")
			{
				return;
			}
			
			%brickSizeX  = getRecord (%brickData, 0);
			%brickSizeY  = getRecord (%brickData, 1);
			%brickSizeZ  = getRecord (%brickData, 2);
			%texName     = getRecord (%data, 0);
			%drag        = getRecord (%data, 1);
			%gravity     = getRecord (%data, 2);
			%lifetime    = getRecord (%data, 3);
			%colorStart  = getRecord (%data, 4);
			%colorEnd    = getRecord (%data, 5);
			%sizeStart   = getRecord (%data, 6);
			%sizeEnd     = getRecord (%data, 7);
			%velocity    = getRecord (%data, 8);
			%periodMS    = getRecord (%data, 9);
			%useEmColors = getRecord (%data, 10);
			%velVariance = getRecord (%data, 11);
			%uiName      = getSafeVariableName (%uiName);
			
			if (%SO.hasParticleType[%uiName] $= "")
			{
				%SO.hasParticleType[%uiName] = 1;
				%SO.writeLine ("PARTICLETYPE " @ %texName @ "\" " @ %uiName @ " " @ atof (%gravity) @ " " @ atof (%drag));
			}
			
			%SO.writeLine ("BEGIN_EMITTER");
			%SO.writeLine ("\tTYPE " @ %uiName);
			%SO.writeLine ("\tPOS " @ %SO.lastBrickPos);
			%SO.writeLine ("\tLIFETIME " @ %lifetime);
			%SO.writeLine ("\tVEL_MIN " @ (%velocity - %velVariance) SPC (%velocity - %velVariance) SPC (%velocity - %velVariance));
			%SO.writeLine ("\tVEL_MAX " @ (%velocity + %velVariance) SPC (%velocity + %velVariance) SPC (%velocity + %velVariance));
			%SO.writeLine ("\tINIT_SCALE_MIN " @ %sizeStart SPC %sizeStart SPC %sizeStart);
			%SO.writeLine ("\tINIT_SCALE_MAX " @ %sizeStart SPC %sizeStart SPC %sizeStart);
			%SO.writeLine ("\tEND_SCALE_MIN " @ %sizeEnd SPC %sizeEnd SPC %sizeEnd);
			%SO.writeLine ("\tEND_SCALE_MAX " @ %sizeEnd SPC %sizeEnd SPC %sizeEnd);
			%SO.writeLine ("\tINIT_ALPHA " @ getWord (%colorStart, 3));
			%SO.writeLine ("\tEND_ALPHA " @ getWord (%colorEnd, 3));
			%SO.writeLine ("\tCOLOR " @ (%useEmColors ? %SO.lastBrickColor : %colorStart));
			%SO.writeLine ("\tSPAWN_OFFSET " @ ((%brickSizeX > %brickSizeY ? %brickSizeX : %brickSizeY) / 2));
			%SO.writeLine ("\tEJECT_PERIOD_MS " @ %periodMS);
			%SO.writeLine ("END_EMITTER");
			
		case "item":
			%data = $MMBuildShowcase::UINameTable["item", %uiName];
			
			if (%data $= "")
			{
				error ("ERROR: bls2blrs() - Failed to find item datablock with uiname \"" @ %uiName @ "\"");
				return;
			}
			
			%brickData = $MMBuildShowcase::UINameTable["brick", %SO.lastBrickUIName];
			if (%brickData $= "")
			{
				return;
			}
			
			%brickSizeX   = getRecord (%brickData, 0);
			%brickSizeY   = getRecord (%brickData, 1);
			%brickSizeZ   = getRecord (%brickData, 2);
			%shapeFile    = getRecord (%data, 0);
			%doColorShift = getRecord (%data, 1);
			%colorShift   = getRecord (%data, 2);
			
			%SO.shapeCount++;
			
			%SO.writeLine ("BEGIN_SHAPE");
			%SO.writeLine ("\tNAME item" @ %SO.shapeCount);
			%SO.writeLine ("\tMODEL " @ %shapeFile);
			%SO.writeLine ("\tPOSITION " @ getWords (%SO.lastBrickPos, 0, 1) SPC (getWord (%SO.lastBrickPos, 2) + (%brickSizeZ * 0.04)));
			%SO.writeLine ("\tSCALE 1 1 1");
			%SO.writeLine ("\tROTATION 0");
			%SO.writeLine ("\tABS_POS 0");
			%SO.writeLine ("END_SHAPE");
			
			if (%doColorShift)
			{
				%SO.writeLine ("SET_NODE_COLOR\titem" @ %SO.shapeCount @ "\tALL\t" @ %colorShift);
			}
	}
}

function bls2blrs (%inputFile)
{
	if (!isFile (%inputFile))
	{
		error("ERROR: bls2blrs() - Failed to open output file \"" @ %inputFile @ "\"");
		return;
	}
	
	echo ("Loading BLRS conversion table...");
	BLRS_LoadConversionTable ();
	
	%outputFile = "config/client/buildShowcase/builds/" @ fileBase (%inputFile) @ ".blrs";
	
	// Log this
	echo ("Converting BLS file \"" @ %inputFile @ "\" to BLRS (Output to \"" @ %outputFile @ "\")...");
	
	// Open the input file
	%SO = new FileObject ();
	if (!%SO.openForRead (%inputFile))
	{
		error ("ERROR: bls2blrs() - Failed to open output file \"" @ %inputFile @ "\"");
		%SO.delete ();
		return;
	}
	
	// Open the output file
	%OutSO = new FileObject ()
	{
		bounds          = "0 0 0 0 0 0";
		shapeCount      = 0;
		validBrickCount = 0;
	};
	if (!%OutSO.openForWrite (%outputFile))
	{
		error ("ERROR: bls2blrs() - Failed to open output file \"" @ %outputFile @ "\"");
		%SO.close ();
		%SO.delete ();
		%OutSO.delete ();
		return;
	}
	
	// Get some stats ready
	%emCount   = 0;		//<< Emitter Count
	%ptCount   = 0;		//<< Particle Count
	%lastBrick = new SimObject ();
	
	// Read the header information
	if (%SO.readLine () !$= "This is a Blockland save file.  You probably shouldn't modify it cause you'll screw it up.")
	{
		error("ERROR: bls2blrs() - BLS file \"" @ %inputFile @ "\" is not a valid Blockland save file (Header is invalid)!");
		%SO.close ();
		%SO.delete ();
		%OutSO.close ();
		%OutSO.delete ();
		%lastBrick.delete ();
		return;
	}
	
	// Write the header
	%OutSO.writeLine ("This is a Blockland Rebuilt background save file.  You probably shouldn't modify it cause you'll screw it up.");
	
	// Write the description
	%descLength = atoi (%SO.readLine ());
	%OutSO.writeLine (%descLength);
	while (%descLength)
	{
		if (%SO.isEOF ())
		{
			break;
		}
		
		%OutSO.writeLine (%SO.readLine ());
		%descLength--;
	}
	
	// Read the colorset
	for (%i = 0; %i < 64; %i++)
	{
		%OutSO.colorSet[%i] = %SO.readLine ();
		%OutSO.writeLine (%OutSO.colorSet[%i]);
	}
	
	// Write this highly important & essential line:
	%OutSO.writeLine ("Begin");
	
	// Validate again
	%LineCount = %SO.readLine ();
	if (strPos (%LineCount, "Linecount") == -1)
	{
		error ("ERROR: bls2blrs() - BLS file \"" @ %inputFile @ "\" is not a valid Blockland save file (Failed to read 'Linecount')!");
		%SO.close ();
		%SO.delete ();
		%OutSO.close ();
		%OutSO.delete ();
		%lastBrick.delete ();
		return;
	}
	
	// Read all brick lines
	while (!%SO.isEOF ())
	{
		%line = %SO.readLine ();
		
		if (getSubStr (%line, 0, 2) $= "+-")
		{
			if (firstWord (%line) $= "+-EMITTER")
			{
				%uiName = removeWord (%line, 0);
				%uiName = getSubStr (%uiName, 0, strPos (%uiName, "\""));
				BLRS_ConstructData (%OutSO, "emitter", %uiName, %line);
			}
			else if (firstWord (%line) $= "+-ITEM")
			{
				%uiName = removeWord (%line, 0);
				%uiName = getSubStr (%uiName, 0, strPos (%uiName, "\""));
				BLRS_ConstructData (%OutSO, "item", %uiName, %line);
			}
			
			continue;
		}
		
		if (strPos (%line, "\"") == -1)
			continue;
		
		%uiName = getSubStr (%line, 0, strPos (%line, "\""));
		BLRS_ConstructData (%OutSO, "brick", %uiName, %line);
	}
	
	// Calculate a good orbit point
	if (%OutSO.validBrickCount != 0)
	{
		%OutSO.boundsCenter = VectorScale (%OutSO.boundsCenter, 1.0 / %OutSO.validBrickCount);
		%OutSO.writeLine ("ORBIT_POS" SPC VectorSub (BoxCenter (%OutSO.bounds), %OutSO.boundsCenter));
		%OutSO.writeLine ("CAMERA_DIST" SPC 300 * 4 / VectorLen (BoxLen (%OutSO.bounds)));
	}
	
	%OutSO.writeLine ("ALLOW_SKY Blue2 1");
	%OutSO.writeLine ("ALLOW_SKY Spooky1 1");
	%OutSO.writeLine ("ALLOW_SKY Skylands 1");
	%OutSO.writeLine ("ALLOW_SKY Sunset 1");
	
	%SO.close ();
	%SO.delete ();
	%OutSO.close ();
	%OutSO.delete ();
	
	deleteVariables ("$MMBuildShowcase::UINameTable*");
	
	if (isObject (mainMenuGui) && isObject (MM_BuildPreview) && MM_BuildPreview.isAwake ())
		MM_BuildPreview.loadBackgroundBuild (fileBase (%inputFile));
}